
class Person(object):
    
    male_codes = set(['1', 'm', 'M', 'male'])
    female_codes = set(['2', 'f', 'F', 'female'])
    unknown_codes = set(['0', 'NA', 'unknown', '.', '-9'])
    
    def __init__(self, family, id, dad, mom, sex, phenotype):
        
        self.family = family
        self.id = id
        self.mom = mom
        self.dad = dad
        self.sex = sex
        self.phenotype = phenotype
        
        if self.sex not in self.male_codes | self.female_codes | self.unknown_codes:
            raise ValueError('unknown sex code: {}'.format(self.sex))
        
        if self.phenotype not in set(['1', '2']) | self.unknown_codes:
            raise ValueError('unknown phenotype: {}'.format(self.phenotype))
    
    def __repr__(self):
        return 'Person(family="{}", person="{}", mom="{}", dad="{}", sex="{}", ' \
            'phenotype="{}")'.format(self.family, self.id, self.mom,
            self.dad, self.sex, self.phenotype)
    
    def __str__(self):
        return self.id
    
    def __hash__(self):
        return hash((self.family, self.id))
    
    def __eq__(self, other):
        return hash(self) == hash(other)
    
    def is_affected(self):
        return self.phenotype == "2"
    
    def is_male(self):
        return self.sex in self.male_codes
    
    def unknown_sex(self):
        return self.sex in self.unknown_codes




import networkx as nx

class Family(nx.DiGraph):
    """ this is mostly a wrapper around networkx's DiGraph class
    
    Family members are stored as nodes, and children are linked to their parents
    by edges. More distant relatives must be identified by traversing the graph
    via child -> parent -> grandparent etc.
    """
    
    def __init__(self, id):
        self.id = id
        
        # properly initialise the networkx DiGraph class
        super(Family, self).__init__()
    
    def __repr__(self):
        return 'Family({})'.format(self.id)
    
    def get_parents(self, person):
        return self.predecessors(person)
    
    def get_father(self, person):
        # return the male parent, or None if father not present
        for x in self.get_parents(person):
            if x.is_male():
                return x
    
    def get_mother(self, person):
        # return the female parent, or None if mother not present
        for x in self.get_parents(person):
            if x.is_female():
                return x
    
    def get_children(self, person):
        return self.successors(person)
    
    def add_person(self, person):
        self.add_node(person)
    
    def set_mom(self, mom, child):
        self.add_edge(mom, child)
        # get the actual node for the mom, which contains the sex
        nodes = list(self.nodes)
        if nodes[nodes.index(mom)].is_male():
            raise ValueError("mom is not female: {}".format(mom.id))
    
    def set_dad(self, dad, child):
        self.add_edge(dad, child)
        # get the actual node for the dad, which contains the sex
        nodes = list(self.nodes)
        dad = nodes[nodes.index(dad)]
        if not (dad.is_male() or dad.unknown_sex()):
            raise ValueError("dad is not male: {}".format(dad.id))
    
    def get_proband(self):
        for x in self.nodes():
            if x.is_affected():
                return x




from peds.person import Person
from peds.family import Family

def open_ped(path):
    """ open a ped file and return a list of Family objects
    """
    
    # split file into lists of lines for each family, indexed by family ID
    fams = {}
    with open(path) as handle:
        sep = get_separator(handle)
        for line in handle:
            # ignore header and comment lines
            if line.startswith('family_id\t') or line.startswith('#'):
                continue
            
            fam_id = line.split('\t', 1)[0]
            if fam_id not in fams:
                fams[fam_id] = Family(fam_id)
            
            person = Person(*line.strip().split('\t'))
            if person in fams[fam_id]:
                raise ValueError('already family: {}'.format(person))
            
            fams[fam_id].add_person(person)
    
    return [ link_members(x) for x in list(fams.values()) ]

def get_separator(handle):
    """ get the column separator (assumes same on all lines)
    """
    current = handle.tell()
    line = handle.readline()
    handle.seek(current)
    
    if line.count('\t') > line.count(' '):
        return '\t'
    else:
        return ' '

def link_members(family):
    """ links family members, i.e. parents to children
    """
    
    # link parents to their children
    for person in family:
        # make placeholder parents, to match on family and individual IDs
        mom = Person(family.id, person.mom, 'NA', 'NA', 'NA', 'NA')
        dad = Person(family.id, person.dad, 'NA', 'NA', 'NA', 'NA')
        
        if mom.id != '0' and mom in family:
            family.set_mom(mom, person)
        
        if dad.id != '0' and dad in family:
            family.set_dad(dad, person)
    
    return family




from setuptools import setup

setup (name="peds",
    description='Package for parsing pedigree files',
    version="1.0.0",
    author="Jeremy McRae",
    author_email="jmcrae@illumina.com",
    url='https://github.com/jeremymcrae/peds',
    packages=["peds"],
    install_requires=['networkx',
    ],
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Topic :: Scientific/Engineering :: Bio-Informatics",
    ])
